from .baseline_base import BaseLineBase


class BaselineNumerical(BaseLineBase):
    def __init__(self, x, y):
        """
        Numerical baseline

        Parameters
        ----------
        x : array-like
        y : array-like
        """
