#!/usr/bin/env python3

from .baseline import *  # noqa
from .peak_profile import *  # noqa
from .spectral_component_collection import SpectralComponentCollection  # noqa

#  raise DeprecationWarning('This module is deprecated')
